﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Pong.Behaviours;
using Syntriax.Engine.Core;
using Syntriax.Engine.Core.Abstract;
using Syntriax.Engine.Graphics.TwoDimensional;
using Syntriax.Engine.Input;

namespace Pong;

public class Game1 : Game
{
    private GraphicsDeviceManager _graphics;
    private SpriteBatch _spriteBatch;
    private GameManager gameManager = null!;

    public Game1()
    {
        _graphics = new GraphicsDeviceManager(this);
        Content.RootDirectory = "Content";
        IsMouseVisible = true;
    }

    protected override void Initialize()
    {
        gameManager = new();

        // TODO: Add your initialization logic here
        gameManager.Initialize();

        base.Initialize();
    }

    protected override void LoadContent()
    {
        _spriteBatch = new SpriteBatch(GraphicsDevice);

        Sprite spriteBox = new Sprite() { Texture2D = Content.Load<Texture2D>("Sprites/Pixel") };
        Sprite spriteBall = new Sprite() { Texture2D = Content.Load<Texture2D>("Sprites/Circle") };

        IGameObject gameObjectCamera = gameManager.InstantiateGameObject<GameObject>();
        gameObjectCamera.Name = "Camera";
        gameObjectCamera.Transform.Position = Vector2.Zero;

        gameManager.Camera = gameObjectCamera.BehaviourController.AddBehaviour<CameraBehaviour>();
        gameManager.Camera.Viewport = GraphicsDevice.Viewport;

        IGameObject gameObjectBall = gameManager.InstantiateGameObject<GameObject>();
        gameObjectBall.Name = "Ball";
        gameObjectBall.Transform.Position = Vector2.Zero;
        gameObjectBall.Transform.Scale = new Vector2(1f / 51.2f, 1f / 51.2f);
        // gameObjectBall.BehaviourController.AddBehaviour<MovementBallBehaviour>(new Vector2(.1f, .1f), 100f);
        gameObjectBall.BehaviourController.AddBehaviour<DisplayableSpriteBehaviour>().Assign(spriteBall);

        IGameObject gameObjectLeft = gameManager.InstantiateGameObject<GameObject>();
        gameObjectLeft.Name = "Left";
        gameObjectLeft.Transform.Position = new Vector2(-350f, 0f);
        gameObjectLeft.Transform.Scale = new Vector2(10f, 40f);
        gameObjectLeft.BehaviourController.AddBehaviour<KeyboardInputsBehaviour>();
        gameObjectLeft.BehaviourController.AddBehaviour<MovementBoxBehaviour>(Keys.W, Keys.S, 400f);
        gameObjectLeft.BehaviourController.AddBehaviour<DisplayableSpriteBehaviour>().Assign(spriteBox);

        IGameObject gameObjectRight = gameManager.InstantiateGameObject<GameObject>();
        gameObjectRight.Name = "Right";
        gameObjectRight.Transform.Position = new Vector2(350f, 0f);
        gameObjectRight.Transform.Scale = new Vector2(10f, 40f);
        gameObjectRight.BehaviourController.AddBehaviour<KeyboardInputsBehaviour>();
        gameObjectRight.BehaviourController.AddBehaviour<MovementBoxBehaviour>(Keys.Up, Keys.Down, 400f);
        gameObjectRight.BehaviourController.AddBehaviour<DisplayableSpriteBehaviour>().Assign(spriteBox);
        // TODO: use this.Content to load your game content here
    }

    protected override void Update(GameTime gameTime)
    {
        if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
            Exit();

        if (Keyboard.GetState().IsKeyDown(Keys.F))
        {
            if (_graphics.IsFullScreen)
                return;

            _graphics.PreferMultiSampling = false;
            _graphics.PreferredBackBufferWidth = GraphicsDevice.Adapter.CurrentDisplayMode.Width;
            _graphics.PreferredBackBufferHeight = GraphicsDevice.Adapter.CurrentDisplayMode.Height;
            _graphics.IsFullScreen = true;
            _graphics.ApplyChanges();

            float previousScreenSize = MathF.Sqrt(MathF.Pow(gameManager.Camera.Viewport.Width, 2f) + MathF.Pow(gameManager.Camera.Viewport.Height, 2f));
            float currentScreenSize = MathF.Sqrt(MathF.Pow(GraphicsDevice.Viewport.Width, 2f) + MathF.Pow(GraphicsDevice.Viewport.Height, 2f));
            gameManager.Camera.Zoom /= previousScreenSize / currentScreenSize;
            gameManager.Camera.Viewport = GraphicsDevice.Viewport;
        }

        if (Keyboard.GetState().IsKeyDown(Keys.U))
            gameManager.Camera.Zoom += gameTime.ElapsedGameTime.Nanoseconds * 0.00025f;
        if (Keyboard.GetState().IsKeyDown(Keys.J))
            gameManager.Camera.Zoom -= gameTime.ElapsedGameTime.Nanoseconds * 0.00025f;

        if (Keyboard.GetState().IsKeyDown(Keys.Q))
            gameManager.Camera.Rotation += gameTime.ElapsedGameTime.Nanoseconds * 0.000025f;
        if (Keyboard.GetState().IsKeyDown(Keys.E))
            gameManager.Camera.Rotation -= gameTime.ElapsedGameTime.Nanoseconds * 0.000025f;

        // TODO: Add your update logic here
        gameManager.Update(gameTime);

        base.Update(gameTime);
    }

    protected override void Draw(GameTime gameTime)
    {
        GraphicsDevice.Clear(new Color() { R = 32, G = 32, B = 32 });

        // TODO: Add your drawing code here
        gameManager.PreDraw(gameTime);
        gameManager.Camera.Update();
        gameManager.Draw(_spriteBatch);

        base.Draw(gameTime);
    }
}
